@@ -28,7 +28,6 @@
  */
 
 #include "sysdeps.h"
-#include <string.h>
 #include "gstvaapiutils.h"
 #include "gstvaapivalue.h"
 #include "gstvaapidisplay.h"
@@ -52,11 +51,6 @@ GST_DEBUG_CATEGORY (gst_debug_vaapi_display);
 G_DEFINE_TYPE_WITH_CODE (GstVaapiDisplay, gst_vaapi_display, GST_TYPE_OBJECT,
     _do_init);
 
-/* Ensure those symbols are actually defined in the resulting libraries */
-#undef gst_vaapi_display_ref
-#undef gst_vaapi_display_unref
-#undef gst_vaapi_display_replace
-
 typedef struct _GstVaapiConfig GstVaapiConfig;
 struct _GstVaapiConfig
 {
@@ -469,12 +463,8 @@ ensure_profiles (GstVaapiDisplay * display)
   VAStatus status;
   gboolean success = FALSE;
 
-  GST_VAAPI_DISPLAY_LOCK (display);
-
-  if (priv->has_profiles) {
-    GST_VAAPI_DISPLAY_UNLOCK (display);
+  if (priv->has_profiles)
     return TRUE;
-  }
 
   priv->decoders = g_array_new (FALSE, FALSE, sizeof (GstVaapiConfig));
   if (!priv->decoders)
@@ -542,7 +532,6 @@ ensure_profiles (GstVaapiDisplay * display)
   g_array_sort (priv->encoders, compare_profiles);
 
   /* Video processing API */
-#if USE_VA_VPP
   status = vaQueryConfigEntrypoints (priv->display, VAProfileNone,
       entrypoints, &num_entrypoints);
   if (vaapi_check_status (status, "vaQueryEntrypoints() [VAProfileNone]")) {
@@ -551,13 +540,11 @@ ensure_profiles (GstVaapiDisplay * display)
         priv->has_vpp = TRUE;
     }
   }
-#endif
   success = TRUE;
 
 cleanup:
   g_free (profiles);
   g_free (entrypoints);
-  GST_VAAPI_DISPLAY_UNLOCK (display);
   return success;
 }
 
@@ -856,17 +843,14 @@ gst_vaapi_display_destroy (GstVaapiDisplay * display)
       klass->close_display (display);
   }
 
-  g_free (priv->display_name);
-  priv->display_name = NULL;
+  g_clear_pointer (&priv->display_name, g_free);
+  g_clear_pointer (&priv->vendor_string, g_free);
 
-  g_free (priv->vendor_string);
-  priv->vendor_string = NULL;
-
-  gst_vaapi_display_replace_internal (&priv->parent, NULL);
+  gst_vaapi_display_replace (&priv->parent, NULL);
 }
 
 static gboolean
-gst_vaapi_display_create_unlocked (GstVaapiDisplay * display,
+gst_vaapi_display_create (GstVaapiDisplay * display,
     GstVaapiDisplayInitType init_type, gpointer data)
 {
   GstVaapiDisplayPrivate *const priv = GST_VAAPI_DISPLAY_GET_PRIVATE (display);
@@ -920,13 +904,6 @@ gst_vaapi_display_create_unlocked (GstVaapiDisplay * display,
   return TRUE;
 }
 
-static gboolean
-gst_vaapi_display_create (GstVaapiDisplay * display,
-    GstVaapiDisplayInitType init_type, gpointer init_value)
-{
-  return gst_vaapi_display_create_unlocked (display, init_type, init_value);
-}
-
 static void
 gst_vaapi_display_lock_default (GstVaapiDisplay * display)
 {
@@ -998,7 +975,7 @@ _set_property (GstVaapiDisplay * display, const GstVaapiProperty * prop,
 }
 
 static void
-_gst_vaapi_display_set_property (GObject * object, guint property_id,
+gst_vaapi_display_set_property (GObject * object, guint property_id,
     const GValue * value, GParamSpec * pspec)
 {
   GstVaapiDisplay *display = GST_VAAPI_DISPLAY (object);
@@ -1058,7 +1035,7 @@ _get_property (GstVaapiDisplay * display, const GstVaapiProperty * prop,
 }
 
 static void
-_gst_vaapi_display_get_property (GObject * object, guint property_id,
+gst_vaapi_display_get_property (GObject * object, guint property_id,
     GValue * value, GParamSpec * pspec)
 {
   GstVaapiDisplay *display = GST_VAAPI_DISPLAY (object);
@@ -1094,8 +1071,8 @@ gst_vaapi_display_class_init (GstVaapiDisplayClass * klass)
   GObjectClass *const object_class = G_OBJECT_CLASS (klass);
 
   object_class->finalize = gst_vaapi_display_finalize;
-  object_class->set_property = _gst_vaapi_display_set_property;
-  object_class->get_property = _gst_vaapi_display_get_property;
+  object_class->set_property = gst_vaapi_display_set_property;
+  object_class->get_property = gst_vaapi_display_get_property;
 
   klass->lock = gst_vaapi_display_lock_default;
   klass->unlock = gst_vaapi_display_unlock_default;
@@ -1168,11 +1145,24 @@ gst_vaapi_display_class_init (GstVaapiDisplayClass * klass)
   g_object_class_install_properties (object_class, N_PROPERTIES, g_properties);
 }
 
+/**
+ * gst_vaapi_display_config:
+ * @display: instance of #GstVaapiDisplay
+ * @init_type: type of initialization #GstVaapiDisplayInitType
+ * @init_value: a pointer to the structure with the initialization
+ * parameters
+ *
+ * Binds @display to the VA layer; otherwise it is just an empty
+ * structure.
+ *
+ * Returns: the configured @display if it was configured correctly;
+ * otherwise unrefs @display and returns %NULL.
+ **/
 GstVaapiDisplay *
-gst_vaapi_display_new (GstVaapiDisplay * display,
+gst_vaapi_display_config (GstVaapiDisplay * display,
     GstVaapiDisplayInitType init_type, gpointer init_value)
 {
-  g_return_val_if_fail (display != NULL, NULL);
+  g_return_val_if_fail (display && GST_VAAPI_IS_DISPLAY (display), NULL);
 
   if (!gst_vaapi_display_create (display, init_type, init_value))
     goto error;
@@ -1181,7 +1171,7 @@ gst_vaapi_display_new (GstVaapiDisplay * display,
   /* ERRORS */
 error:
   {
-    gst_vaapi_display_unref_internal (display);
+    gst_object_unref (display);
     return NULL;
   }
 }
@@ -1202,37 +1192,10 @@ gst_vaapi_display_new_with_display (VADisplay va_display)
     .va_display = va_display,
   };
 
-  return gst_vaapi_display_new (g_object_new (GST_TYPE_VAAPI_DISPLAY, NULL),
+  return gst_vaapi_display_config (g_object_new (GST_TYPE_VAAPI_DISPLAY, NULL),
       GST_VAAPI_DISPLAY_INIT_FROM_VA_DISPLAY, &info);
 }
 
-/**
- * gst_vaapi_display_ref:
- * @display: a #GstVaapiDisplay
- *
- * Atomically increases the reference count of the given @display by one.
- *
- * Returns: The same @display argument
- */
-GstVaapiDisplay *
-gst_vaapi_display_ref (GstVaapiDisplay * display)
-{
-  return gst_vaapi_display_ref_internal (display);
-}
-
-/**
- * gst_vaapi_display_unref:
- * @display: a #GstVaapiDisplay
- *
- * Atomically decreases the reference count of the @display by one. If
- * the reference count reaches zero, the display will be free'd.
- */
-void
-gst_vaapi_display_unref (GstVaapiDisplay * display)
-{
-  gst_vaapi_display_unref_internal (display);
-}
-
 /**
  * gst_vaapi_display_replace:
  * @old_display_ptr: a pointer to a #GstVaapiDisplay
@@ -1246,7 +1209,8 @@ void
 gst_vaapi_display_replace (GstVaapiDisplay ** old_display_ptr,
     GstVaapiDisplay * new_display)
 {
-  gst_vaapi_display_replace_internal (old_display_ptr, new_display);
+  gst_object_replace ((GstObject **) old_display_ptr,
+      (GstObject *) new_display);
 }
 
 /**
@@ -1744,48 +1708,6 @@ gst_vaapi_display_has_property (GstVaapiDisplay * display, const gchar * name)
       name) != NULL;
 }
 
-gboolean
-gst_vaapi_display_get_property (GstVaapiDisplay * display, const gchar * name,
-    GValue * out_value)
-{
-  const GstVaapiProperty *prop;
-
-  g_return_val_if_fail (display != NULL, FALSE);
-  g_return_val_if_fail (name != NULL, FALSE);
-  g_return_val_if_fail (out_value != NULL, FALSE);
-
-  if (!ensure_properties (display))
-    return FALSE;
-
-  prop =
-      find_property (GST_VAAPI_DISPLAY_GET_PRIVATE (display)->properties, name);
-  if (!prop)
-    return FALSE;
-
-  return _get_property (display, prop, out_value);
-}
-
-gboolean
-gst_vaapi_display_set_property (GstVaapiDisplay * display, const gchar * name,
-    const GValue * value)
-{
-  const GstVaapiProperty *prop;
-
-  g_return_val_if_fail (display != NULL, FALSE);
-  g_return_val_if_fail (name != NULL, FALSE);
-  g_return_val_if_fail (value != NULL, FALSE);
-
-  if (!ensure_properties (display))
-    return FALSE;
-
-  prop =
-      find_property (GST_VAAPI_DISPLAY_GET_PRIVATE (display)->properties, name);
-  if (!prop)
-    return FALSE;
-
-  return _set_property (display, prop, value);
-}
-
 static gboolean
 get_attribute (GstVaapiDisplay * display, VADisplayAttribType type,
     gint * value)
