@@ -27,7 +27,6 @@
  */
 
 #include "sysdeps.h"
-#include <string.h>
 #include "gstvaapidisplay_priv.h"
 #include "gstvaapidisplay_wayland.h"
 #include "gstvaapidisplay_wayland_priv.h"
@@ -36,11 +35,8 @@
 #define DEBUG_VAAPI_DISPLAY 1
 #include "gstvaapidebug.h"
 
-#define _do_init \
-    G_ADD_PRIVATE (GstVaapiDisplayWayland);
-
-G_DEFINE_TYPE_WITH_CODE (GstVaapiDisplayWayland, gst_vaapi_display_wayland,
-    GST_TYPE_VAAPI_DISPLAY, _do_init);
+G_DEFINE_TYPE_WITH_PRIVATE (GstVaapiDisplayWayland, gst_vaapi_display_wayland,
+    GST_TYPE_VAAPI_DISPLAY);
 
 static inline const gchar *
 get_default_display_name (void)
@@ -194,25 +190,10 @@ gst_vaapi_display_wayland_close_display (GstVaapiDisplay * display)
   GstVaapiDisplayWaylandPrivate *const priv =
       GST_VAAPI_DISPLAY_WAYLAND_GET_PRIVATE (display);
 
-  if (priv->output) {
-    wl_output_destroy (priv->output);
-    priv->output = NULL;
-  }
-
-  if (priv->shell) {
-    wl_shell_destroy (priv->shell);
-    priv->shell = NULL;
-  }
-
-  if (priv->compositor) {
-    wl_compositor_destroy (priv->compositor);
-    priv->compositor = NULL;
-  }
-
-  if (priv->registry) {
-    wl_registry_destroy (priv->registry);
-    priv->registry = NULL;
-  }
+  g_clear_pointer (&priv->output, wl_output_destroy);
+  g_clear_pointer (&priv->shell, wl_shell_destroy);
+  g_clear_pointer (&priv->compositor, wl_compositor_destroy);
+  g_clear_pointer (&priv->registry, wl_registry_destroy);
 
   if (priv->wl_display) {
     if (!priv->use_foreign_display)
@@ -220,10 +201,7 @@ gst_vaapi_display_wayland_close_display (GstVaapiDisplay * display)
     priv->wl_display = NULL;
   }
 
-  if (priv->display_name) {
-    g_free (priv->display_name);
-    priv->display_name = NULL;
-  }
+  g_clear_pointer (&priv->display_name, g_free);
 }
 
 static gboolean
@@ -325,9 +303,11 @@ gst_vaapi_display_wayland_class_init (GstVaapiDisplayWaylandClass * klass)
 GstVaapiDisplay *
 gst_vaapi_display_wayland_new (const gchar * display_name)
 {
-  return gst_vaapi_display_new (g_object_new (GST_TYPE_VAAPI_DISPLAY_WAYLAND,
-          NULL), GST_VAAPI_DISPLAY_INIT_FROM_DISPLAY_NAME,
-      (gpointer) display_name);
+  GstVaapiDisplay *display;
+
+  display = g_object_new (GST_TYPE_VAAPI_DISPLAY_WAYLAND, NULL);
+  return gst_vaapi_display_config (display,
+      GST_VAAPI_DISPLAY_INIT_FROM_DISPLAY_NAME, (gpointer) display_name);
 }
 
 /**
@@ -344,10 +324,49 @@ gst_vaapi_display_wayland_new (const gchar * display_name)
 GstVaapiDisplay *
 gst_vaapi_display_wayland_new_with_display (struct wl_display * wl_display)
 {
+  GstVaapiDisplay *display;
+
   g_return_val_if_fail (wl_display, NULL);
 
-  return gst_vaapi_display_new (g_object_new (GST_TYPE_VAAPI_DISPLAY_WAYLAND,
-          NULL), GST_VAAPI_DISPLAY_INIT_FROM_NATIVE_DISPLAY, wl_display);
+  display = g_object_new (GST_TYPE_VAAPI_DISPLAY_WAYLAND, NULL);
+  return gst_vaapi_display_config (display,
+      GST_VAAPI_DISPLAY_INIT_FROM_NATIVE_DISPLAY, wl_display);
+}
+
+/**
+ * gst_vaapi_display_wayland_new_with_va_display:
+ * @va_display: a VADisplay #va_display
+ * @wl_display: an Wayland #wl_display
+ *
+ * Creates a #GstVaapiDisplay based on the VADisplay @va_display and
+ * the Wayland @wl_display display.
+ * The caller still owns the display and must call
+ * wl_display_disconnect() when all #GstVaapiDisplay references are
+ * released.
+ *
+ * Return value: a newly allocated #GstVaapiDisplay object
+ */
+
+GstVaapiDisplay *
+gst_vaapi_display_wayland_new_with_va_display (VADisplay va_display,
+    struct wl_display * wl_display)
+{
+  GstVaapiDisplay *display;
+  GstVaapiDisplayInfo info = {
+    .va_display = va_display,
+    .native_display = wl_display,
+  };
+
+  g_return_val_if_fail (wl_display, NULL);
+
+  display = g_object_new (GST_TYPE_VAAPI_DISPLAY_WAYLAND, NULL);
+  if (!gst_vaapi_display_config (display,
+          GST_VAAPI_DISPLAY_INIT_FROM_VA_DISPLAY, &info)) {
+    gst_object_unref (display);
+    return NULL;
+  }
+
+  return display;
 }
 
 /**
