@@ -110,6 +110,7 @@ struct _GstVaapiEncoderH265
   guint32 luma_height;
   GstClockTime cts_offset;
   gboolean config_changed;
+  gboolean low_delay_b;
 
   /* maximum required size of the decoded picture buffer */
   guint32 max_dec_pic_buffering;
@@ -1166,7 +1167,7 @@ add_packed_vps_header (GstVaapiEncoderH265 * encoder,
   guint32 data_bit_size;
   guint8 *data;
 
-  gst_bit_writer_init (&bs, 128 * 8);
+  gst_bit_writer_init_with_size (&bs, 128, FALSE);
   WRITE_UINT32 (&bs, 0x00000001, 32);   /* start code */
   bs_write_nal_header (&bs, GST_H265_NAL_VPS);
 
@@ -1190,14 +1191,14 @@ add_packed_vps_header (GstVaapiEncoderH265 * encoder,
 
   /* store vps data */
   _check_vps_sps_pps_status (encoder, data + 4, data_bit_size / 8 - 4);
-  gst_bit_writer_clear (&bs, TRUE);
+  gst_bit_writer_reset (&bs);
   return TRUE;
 
   /* ERRORS */
 bs_error:
   {
     GST_WARNING ("failed to write VPS NAL unit");
-    gst_bit_writer_clear (&bs, TRUE);
+    gst_bit_writer_reset (&bs);
     return FALSE;
   }
 }
@@ -1220,7 +1221,7 @@ add_packed_sequence_header (GstVaapiEncoderH265 * encoder,
 
   fill_hrd_params (encoder, &hrd_params);
 
-  gst_bit_writer_init (&bs, 128 * 8);
+  gst_bit_writer_init_with_size (&bs, 128, FALSE);
   WRITE_UINT32 (&bs, 0x00000001, 32);   /* start code */
   bs_write_nal_header (&bs, GST_H265_NAL_SPS);
 
@@ -1244,14 +1245,14 @@ add_packed_sequence_header (GstVaapiEncoderH265 * encoder,
 
   /* store sps data */
   _check_vps_sps_pps_status (encoder, data + 4, data_bit_size / 8 - 4);
-  gst_bit_writer_clear (&bs, TRUE);
+  gst_bit_writer_reset (&bs);
   return TRUE;
 
   /* ERRORS */
 bs_error:
   {
     GST_WARNING ("failed to write SPS NAL unit");
-    gst_bit_writer_clear (&bs, TRUE);
+    gst_bit_writer_reset (&bs);
     return FALSE;
   }
 }
@@ -1269,7 +1270,7 @@ add_packed_picture_header (GstVaapiEncoderH265 * encoder,
   guint32 data_bit_size;
   guint8 *data;
 
-  gst_bit_writer_init (&bs, 128 * 8);
+  gst_bit_writer_init_with_size (&bs, 128, FALSE);
   WRITE_UINT32 (&bs, 0x00000001, 32);   /* start code */
   bs_write_nal_header (&bs, GST_H265_NAL_PPS);
   bs_write_pps (&bs, pic_param);
@@ -1291,14 +1292,14 @@ add_packed_picture_header (GstVaapiEncoderH265 * encoder,
 
   /* store pps data */
   _check_vps_sps_pps_status (encoder, data + 4, data_bit_size / 8 - 4);
-  gst_bit_writer_clear (&bs, TRUE);
+  gst_bit_writer_reset (&bs);
   return TRUE;
 
   /* ERRORS */
 bs_error:
   {
     GST_WARNING ("failed to write PPS NAL unit");
-    gst_bit_writer_clear (&bs, TRUE);
+    gst_bit_writer_reset (&bs);
     return FALSE;
   }
 }
@@ -1339,7 +1340,7 @@ add_packed_slice_header (GstVaapiEncoderH265 * encoder,
   guint8 *data;
   guint8 nal_unit_type;
 
-  gst_bit_writer_init (&bs, 128 * 8);
+  gst_bit_writer_init_with_size (&bs, 128, FALSE);
   WRITE_UINT32 (&bs, 0x00000001, 32);   /* start code */
 
   if (!get_nal_unit_type (picture, &nal_unit_type))
@@ -1362,14 +1363,14 @@ add_packed_slice_header (GstVaapiEncoderH265 * encoder,
   gst_vaapi_enc_slice_add_packed_header (slice, packed_slice);
   gst_vaapi_codec_object_replace (&packed_slice, NULL);
 
-  gst_bit_writer_clear (&bs, TRUE);
+  gst_bit_writer_reset (&bs);
   return TRUE;
 
   /* ERRORS */
 bs_error:
   {
     GST_WARNING ("failed to write Slice NAL unit header");
-    gst_bit_writer_clear (&bs, TRUE);
+    gst_bit_writer_reset (&bs);
     return FALSE;
   }
 }
@@ -1684,6 +1685,9 @@ add_slice_headers (GstVaapiEncoderH265 * encoder, GstVaapiEncPicture * picture,
     }
     slice_param->num_ctu_in_slice = cur_slice_ctus;
     slice_param->slice_type = h265_get_slice_type (picture->type);
+    if (encoder->low_delay_b && slice_param->slice_type == GST_H265_P_SLICE) {
+      slice_param->slice_type = GST_H265_B_SLICE;
+    }
     slice_param->slice_pic_parameter_set_id = 0;
 
     slice_param->slice_fields.bits.num_ref_idx_active_override_flag =
@@ -1717,6 +1721,13 @@ add_slice_headers (GstVaapiEncoderH265 * encoder, GstVaapiEncPicture * picture,
             GST_VAAPI_SURFACE_PROXY_SURFACE_ID (reflist_1[i_ref]->pic);
         slice_param->ref_pic_list1[i_ref].pic_order_cnt = reflist_1[i_ref]->poc;
       }
+    } else if (picture->type == GST_VAAPI_PICTURE_TYPE_P
+        && encoder->low_delay_b) {
+      for (; i_ref < reflist_0_count; ++i_ref) {
+        slice_param->ref_pic_list1[i_ref].picture_id =
+            GST_VAAPI_SURFACE_PROXY_SURFACE_ID (reflist_0[i_ref]->pic);
+        slice_param->ref_pic_list1[i_ref].pic_order_cnt = reflist_0[i_ref]->poc;
+      }
     }
     for (; i_ref < G_N_ELEMENTS (slice_param->ref_pic_list1); ++i_ref) {
       slice_param->ref_pic_list1[i_ref].picture_id = VA_INVALID_SURFACE;
@@ -2023,7 +2034,6 @@ reset_properties (GstVaapiEncoderH265 * encoder)
   GstVaapiH265ReorderPool *reorder_pool;
   GstVaapiH265RefPool *ref_pool;
   guint ctu_size;
-  gboolean ret;
 
   if (encoder->idr_period < base_encoder->keyframe_period)
     encoder->idr_period = base_encoder->keyframe_period;
@@ -2033,10 +2043,9 @@ reset_properties (GstVaapiEncoderH265 * encoder)
   encoder->qp_i = encoder->init_qp;
 
   ctu_size = encoder->ctu_width * encoder->ctu_height;
-  ret = gst_vaapi_encoder_ensure_num_slices (base_encoder, encoder->profile,
-      GST_VAAPI_ENTRYPOINT_SLICE_ENCODE, (ctu_size + 1) / 2,
-      &encoder->num_slices);
-  g_assert (ret);
+  g_assert (gst_vaapi_encoder_ensure_num_slices (base_encoder, encoder->profile,
+          GST_VAAPI_ENTRYPOINT_SLICE_ENCODE, (ctu_size + 1) / 2,
+          &encoder->num_slices));
 
   gst_vaapi_encoder_ensure_max_num_ref_frames (base_encoder, encoder->profile,
       GST_VAAPI_ENTRYPOINT_SLICE_ENCODE);
@@ -2125,6 +2134,54 @@ error:
   }
 }
 
+struct _PendingIterState
+{
+  GstVaapiPictureType pic_type;
+};
+
+static gboolean
+gst_vaapi_encoder_h265_get_pending_reordered (GstVaapiEncoder * base_encoder,
+    GstVaapiEncPicture ** picture, gpointer * state)
+{
+  GstVaapiEncoderH265 *const encoder = GST_VAAPI_ENCODER_H265 (base_encoder);
+  GstVaapiH265ReorderPool *reorder_pool;
+  GstVaapiEncPicture *pic;
+  struct _PendingIterState *iter;
+
+  g_return_val_if_fail (state, FALSE);
+
+  if (!*state) {
+    iter = g_new0 (struct _PendingIterState, 1);
+    iter->pic_type = GST_VAAPI_PICTURE_TYPE_P;
+    *state = iter;
+  } else {
+    iter = *state;
+  }
+
+  *picture = NULL;
+
+  reorder_pool = &encoder->reorder_pool;
+  if (g_queue_is_empty (&reorder_pool->reorder_frame_list))
+    return FALSE;
+
+  pic = g_queue_pop_tail (&reorder_pool->reorder_frame_list);
+  g_assert (pic);
+  if (iter->pic_type == GST_VAAPI_PICTURE_TYPE_P) {
+    set_p_frame (pic, encoder);
+    iter->pic_type = GST_VAAPI_PICTURE_TYPE_B;
+  } else if (iter->pic_type == GST_VAAPI_PICTURE_TYPE_B) {
+    set_b_frame (pic, encoder);
+  } else {
+    GST_WARNING ("Unhandled pending picture type");
+  }
+
+  if (GST_CLOCK_TIME_IS_VALID (pic->frame->pts))
+    pic->frame->pts += encoder->cts_offset;
+
+  *picture = pic;
+  return TRUE;
+}
+
 static GstVaapiEncoderStatus
 gst_vaapi_encoder_h265_flush (GstVaapiEncoder * base_encoder)
 {
@@ -2175,8 +2232,8 @@ gst_vaapi_encoder_h265_get_codec_data (GstVaapiEncoder * base_encoder,
     goto error_map_pps_buffer;
 
   /* Header */
-  gst_bit_writer_init (&bs,
-      (vps_info.size + sps_info.size + pps_info.size + 64) * 8);
+  gst_bit_writer_init_with_size (&bs,
+      (vps_info.size + sps_info.size + pps_info.size + 64), FALSE);
   WRITE_UINT32 (&bs, configuration_version, 8);
   WRITE_UINT32 (&bs, sps_info.data[4], 8);      /* profile_space | tier_flag | profile_idc */
   WRITE_UINT32 (&bs, sps_info.data[5], 32);     /* profile_compatibility_flag [0-31] */
@@ -2237,13 +2294,15 @@ gst_vaapi_encoder_h265_get_codec_data (GstVaapiEncoder * base_encoder,
   gst_buffer_unmap (encoder->sps_data, &sps_info);
   gst_buffer_unmap (encoder->vps_data, &vps_info);
 
-  buffer = gst_buffer_new_wrapped (GST_BIT_WRITER_DATA (&bs),
-      GST_BIT_WRITER_BIT_SIZE (&bs) / 8);
+  buffer = gst_bit_writer_reset_and_get_buffer (&bs);
   if (!buffer)
     goto error_alloc_buffer;
+  if (gst_buffer_n_memory (buffer) == 0) {
+    gst_buffer_unref (buffer);
+    goto error_alloc_buffer;
+  }
   *out_buffer_ptr = buffer;
 
-  gst_bit_writer_clear (&bs, FALSE);
   return GST_VAAPI_ENCODER_STATUS_SUCCESS;
 
   /* ERRORS */
@@ -2253,7 +2312,7 @@ bs_error:
     gst_buffer_unmap (encoder->vps_data, &vps_info);
     gst_buffer_unmap (encoder->sps_data, &sps_info);
     gst_buffer_unmap (encoder->pps_data, &pps_info);
-    gst_bit_writer_clear (&bs, TRUE);
+    gst_bit_writer_reset (&bs);
     return GST_VAAPI_ENCODER_STATUS_ERROR_OPERATION_FAILED;
   }
 nal_to_byte_stream_error:
@@ -2262,7 +2321,7 @@ nal_to_byte_stream_error:
     gst_buffer_unmap (encoder->vps_data, &vps_info);
     gst_buffer_unmap (encoder->sps_data, &sps_info);
     gst_buffer_unmap (encoder->pps_data, &pps_info);
-    gst_bit_writer_clear (&bs, TRUE);
+    gst_bit_writer_reset (&bs);
     return GST_VAAPI_ENCODER_STATUS_ERROR_OPERATION_FAILED;
   }
 error_map_vps_buffer:
@@ -2284,7 +2343,7 @@ error_map_pps_buffer:
 error_alloc_buffer:
   {
     GST_ERROR ("failed to allocate codec-data buffer");
-    gst_bit_writer_clear (&bs, TRUE);
+    gst_bit_writer_reset (&bs);
     return GST_VAAPI_ENCODER_STATUS_ERROR_ALLOCATION_FAILED;
   }
 }
@@ -2439,8 +2498,8 @@ set_context_info (GstVaapiEncoder * base_encoder)
       + (encoder->num_bframes > 0 ? 1 : 0) + DEFAULT_SURFACES_COUNT);
 
   /* Only YUV 4:2:0 formats are supported for now. */
-  base_encoder->codedbuf_size += GST_ROUND_UP_32 (vip->width) *
-      GST_ROUND_UP_32 (vip->height) * 3 / 2;
+  base_encoder->codedbuf_size += GST_ROUND_UP_16 (vip->width) *
+      GST_ROUND_UP_16 (vip->height) * 3 / 2;
 
   return GST_VAAPI_ENCODER_STATUS_SUCCESS;
 }
@@ -2458,15 +2517,15 @@ gst_vaapi_encoder_h265_reconfigure (GstVaapiEncoder * base_encoder)
   if (luma_width != encoder->luma_width || luma_height != encoder->luma_height) {
     GST_DEBUG ("resolution: %d %d", GST_VAAPI_ENCODER_WIDTH (encoder),
         GST_VAAPI_ENCODER_HEIGHT (encoder));
-    encoder->luma_width = GST_ROUND_UP_32 (luma_width);
-    encoder->luma_height = GST_ROUND_UP_32 (luma_height);
+    encoder->luma_width = GST_ROUND_UP_16 (luma_width);
+    encoder->luma_height = GST_ROUND_UP_16 (luma_height);
     encoder->ctu_width = (encoder->luma_width + 31) / 32;
     encoder->ctu_height = (encoder->luma_height + 31) / 32;
     encoder->config_changed = TRUE;
 
     /* Frame Cropping */
-    if ((GST_VAAPI_ENCODER_WIDTH (encoder) & 31) ||
-        (GST_VAAPI_ENCODER_HEIGHT (encoder) & 31)) {
+    if ((GST_VAAPI_ENCODER_WIDTH (encoder) & 15) ||
+        (GST_VAAPI_ENCODER_HEIGHT (encoder) & 15)) {
       static const guint SubWidthC[] = { 1, 2, 2, 1 };
       static const guint SubHeightC[] = { 1, 2, 1, 1 };
       encoder->conformance_window_flag = 1;
@@ -2583,6 +2642,10 @@ gst_vaapi_encoder_h265_set_property (GstVaapiEncoder * base_encoder,
     case GST_VAAPI_ENCODER_H265_PROP_MBBRC:
       encoder->mbbrc = g_value_get_enum (value);
       break;
+    case GST_VAAPI_ENCODER_H265_PROP_LOW_DELAY_B:
+      encoder->low_delay_b = g_value_get_boolean (value);
+      break;
+
     default:
       return GST_VAAPI_ENCODER_STATUS_ERROR_INVALID_PARAMETER;
   }
@@ -2597,7 +2660,8 @@ gst_vaapi_encoder_h265_class (void)
   static const GstVaapiEncoderClass GstVaapiEncoderH265Class = {
     GST_VAAPI_ENCODER_CLASS_INIT (H265, h265),
     .set_property = gst_vaapi_encoder_h265_set_property,
-    .get_codec_data = gst_vaapi_encoder_h265_get_codec_data
+    .get_codec_data = gst_vaapi_encoder_h265_get_codec_data,
+    .get_pending_reordered = gst_vaapi_encoder_h265_get_pending_reordered,
   };
   return &GstVaapiEncoderH265Class;
 }
@@ -2748,6 +2812,18 @@ gst_vaapi_encoder_h265_get_default_properties (void)
           GST_VAAPI_TYPE_ENCODER_MBBRC, GST_VAAPI_ENCODER_MBBRC_AUTO,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  /**
+   * GstVaapiEncoderH265:low_delay_b:
+   *
+   * Enable low delay b frame, which will change P frame with B frame.
+   */
+  GST_VAAPI_ENCODER_PROPERTIES_APPEND (props,
+      GST_VAAPI_ENCODER_H265_PROP_LOW_DELAY_B,
+      g_param_spec_boolean ("low-delay-b",
+          "Enable low delay b",
+          "Transforms P frames into predictive B frames. Enable it when P frames are not supported.",
+          FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   return props;
 }
 
