@@ -24,7 +24,6 @@
 
 #define _GNU_SOURCE 1           /* RTLD_DEFAULT */
 #include "sysdeps.h"
-#include <string.h>
 #include <math.h>
 #include <dlfcn.h>
 #include "gstvaapiutils_glx.h"
@@ -853,6 +852,8 @@ gl_create_pixmap_object (Display * dpy, guint width, guint height)
 
   /* Initialize FBConfig attributes */
   for (attr = fbconfig_attrs; *attr != GL_NONE; attr += 2);
+  *attr++ = GLX_DEPTH_SIZE;
+  *attr++ = wattr.depth;
   if (wattr.depth == 32) {
     *attr++ = GLX_ALPHA_SIZE;
     *attr++ = 8;
