@@ -33,13 +33,23 @@
 #include "gstvaapidisplay_priv.h"
 #include "gstvaapisurface_priv.h"
 
-#define DEBUG 1
-#include "gstvaapidebug.h"
+GST_DEBUG_CATEGORY (gst_debug_vaapi_window);
+#define GST_CAT_DEFAULT gst_debug_vaapi_window
 
-/* Ensure those symbols are actually defined in the resulting libraries */
-#undef gst_vaapi_window_ref
-#undef gst_vaapi_window_unref
-#undef gst_vaapi_window_replace
+#define _do_init \
+    GST_DEBUG_CATEGORY_INIT (gst_debug_vaapi_window, "vaapiwindow", 0, \
+        "VA-API Window");
+
+G_DEFINE_ABSTRACT_TYPE_WITH_CODE (GstVaapiWindow, gst_vaapi_window,
+    GST_TYPE_OBJECT, _do_init);
+
+enum
+{
+  PROP_DISPLAY = 1,
+  PROP_NATIVE_ID,
+  N_PROPERTIES
+};
+static GParamSpec *g_properties[N_PROPERTIES] = { NULL, };
 
 static void
 gst_vaapi_window_ensure_size (GstVaapiWindow * window)
@@ -60,7 +70,7 @@ gst_vaapi_window_ensure_size (GstVaapiWindow * window)
 static gboolean
 ensure_filter (GstVaapiWindow * window)
 {
-  GstVaapiDisplay *const display = GST_VAAPI_OBJECT_DISPLAY (window);
+  GstVaapiDisplay *const display = GST_VAAPI_WINDOW_DISPLAY (window);
 
   /* Ensure VPP pipeline is built */
   if (window->filter)
@@ -92,7 +102,7 @@ error_unsupported_format:
 static gboolean
 ensure_filter_surface_pool (GstVaapiWindow * window)
 {
-  GstVaapiDisplay *const display = GST_VAAPI_OBJECT_DISPLAY (window);
+  GstVaapiDisplay *const display = GST_VAAPI_WINDOW_DISPLAY (window);
 
   if (window->surface_pool)
     goto ensure_filter;
@@ -114,7 +124,7 @@ ensure_filter:
 static gboolean
 gst_vaapi_window_create (GstVaapiWindow * window, guint width, guint height)
 {
-  gst_vaapi_display_get_size (GST_VAAPI_OBJECT_DISPLAY (window),
+  gst_vaapi_display_get_size (GST_VAAPI_WINDOW_DISPLAY (window),
       &window->display_width, &window->display_height);
 
   if (!GST_VAAPI_WINDOW_GET_CLASS (window)->create (window, &width, &height))
@@ -129,24 +139,101 @@ gst_vaapi_window_create (GstVaapiWindow * window, guint width, guint height)
 }
 
 static void
-gst_vaapi_window_finalize (GstVaapiWindow * window)
+gst_vaapi_window_finalize (GObject * object)
 {
+  GstVaapiWindow *const window = GST_VAAPI_WINDOW (object);
+
   gst_vaapi_video_pool_replace (&window->surface_pool, NULL);
   gst_vaapi_filter_replace (&window->filter, NULL);
+  gst_vaapi_display_replace (&window->display, NULL);
+
+  G_OBJECT_CLASS (gst_vaapi_window_parent_class)->finalize (object);
 }
 
-void
+static void
+gst_vaapi_window_set_property (GObject * object, guint property_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstVaapiWindow *const window = GST_VAAPI_WINDOW (object);
+
+  switch (property_id) {
+    case PROP_DISPLAY:
+      g_assert (window->display == NULL);
+      window->display = g_value_dup_object (value);
+      g_assert (window->display != NULL);
+      window->has_vpp = GST_VAAPI_DISPLAY_HAS_VPP (window->display);
+      break;
+    case PROP_NATIVE_ID:{
+      gulong id = g_value_get_ulong (value);
+      window->use_foreign_window = (id != GST_VAAPI_ID_INVALID);
+      GST_VAAPI_WINDOW_ID (window) = window->use_foreign_window ? id : 0;
+      break;
+    }
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+      break;
+  }
+}
+
+static void
+gst_vaapi_window_get_property (GObject * object, guint property_id,
+    GValue * value, GParamSpec * pspec)
+{
+  GstVaapiWindow *const window = GST_VAAPI_WINDOW (object);
+
+  switch (property_id) {
+    case PROP_DISPLAY:
+      g_value_set_object (value, window->display);
+      break;
+    case PROP_NATIVE_ID:
+      g_value_set_ulong (value, window->native_id);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+      break;
+  }
+}
+
+static void
 gst_vaapi_window_class_init (GstVaapiWindowClass * klass)
 {
-  GstVaapiObjectClass *const object_class = GST_VAAPI_OBJECT_CLASS (klass);
+  GObjectClass *const object_class = G_OBJECT_CLASS (klass);
 
-  object_class->finalize = (GstVaapiObjectFinalizeFunc)
-      gst_vaapi_window_finalize;
+  object_class->set_property = gst_vaapi_window_set_property;
+  object_class->get_property = gst_vaapi_window_get_property;
+  object_class->finalize = gst_vaapi_window_finalize;
+
+  /**
+   * GstVaapiWindow:display:
+   *
+   * #GstVaapiDisplay to be used.
+   */
+  g_properties[PROP_DISPLAY] =
+      g_param_spec_object ("display", "Gst VA-API Display",
+      "The VA-API display object to use", GST_TYPE_VAAPI_DISPLAY,
+      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME);
+
+  /**
+   * GstVaapiWindow:native-id:
+   *
+   * Native window ID: either XDisplay, EGLDisplay, or drm-fd.
+   */
+  g_properties[PROP_NATIVE_ID] =
+      g_param_spec_ulong ("native-id", "Native window id",
+      "Native window ID", 0, G_MAXULONG, 0,
+      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME);
+
+  g_object_class_install_properties (object_class, N_PROPERTIES, g_properties);
+}
+
+static void
+gst_vaapi_window_init (GstVaapiWindow * window)
+{
 }
 
 GstVaapiWindow *
-gst_vaapi_window_new_internal (const GstVaapiWindowClass * window_class,
-    GstVaapiDisplay * display, GstVaapiID id, guint width, guint height)
+gst_vaapi_window_new_internal (GType type, GstVaapiDisplay * display,
+    GstVaapiID id, guint width, guint height)
 {
   GstVaapiWindow *window;
 
@@ -158,24 +245,20 @@ gst_vaapi_window_new_internal (const GstVaapiWindowClass * window_class,
     g_return_val_if_fail (height > 0, NULL);
   }
 
-  window = gst_vaapi_object_new (GST_VAAPI_OBJECT_CLASS (window_class),
-      display);
-  if (!window)
-    return NULL;
+  window = g_object_new (type, "display", display, "native-id", id, NULL);
 
-  window->use_foreign_window = id != GST_VAAPI_ID_INVALID;
-  GST_VAAPI_OBJECT_ID (window) = window->use_foreign_window ? id : 0;
-  window->has_vpp =
-      GST_VAAPI_DISPLAY_HAS_VPP (GST_VAAPI_OBJECT_DISPLAY (window));
+  GST_DEBUG_OBJECT (window, "new window with id = 0x%08lx and size %ux%u", id,
+      width, height);
 
   if (!gst_vaapi_window_create (window, width, height))
     goto error;
+
   return window;
 
   /* ERRORS */
 error:
   {
-    gst_vaapi_window_unref_internal (window);
+    gst_object_unref (window);
     return NULL;
   }
 }
@@ -239,7 +322,7 @@ gst_vaapi_window_new (GstVaapiDisplay * display, guint width, guint height)
 {
   GstVaapiDisplayClass *dpy_class;
 
-  g_return_val_if_fail (display != NULL, NULL);
+  g_return_val_if_fail (GST_VAAPI_IS_DISPLAY (display), NULL);
 
   dpy_class = GST_VAAPI_DISPLAY_GET_CLASS (display);
   if (G_UNLIKELY (!dpy_class->create_window))
@@ -248,33 +331,6 @@ gst_vaapi_window_new (GstVaapiDisplay * display, guint width, guint height)
       height);
 }
 
-/**
- * gst_vaapi_window_ref:
- * @window: a #GstVaapiWindow
- *
- * Atomically increases the reference count of the given @window by one.
- *
- * Returns: The same @window argument
- */
-GstVaapiWindow *
-gst_vaapi_window_ref (GstVaapiWindow * window)
-{
-  return gst_vaapi_window_ref_internal (window);
-}
-
-/**
- * gst_vaapi_window_unref:
- * @window: a #GstVaapiWindow
- *
- * Atomically decreases the reference count of the @window by one. If
- * the reference count reaches zero, the window will be free'd.
- */
-void
-gst_vaapi_window_unref (GstVaapiWindow * window)
-{
-  gst_vaapi_window_unref_internal (window);
-}
-
 /**
  * gst_vaapi_window_replace:
  * @old_window_ptr: a pointer to a #GstVaapiWindow
@@ -288,7 +344,7 @@ void
 gst_vaapi_window_replace (GstVaapiWindow ** old_window_ptr,
     GstVaapiWindow * new_window)
 {
-  gst_vaapi_window_replace_internal (old_window_ptr, new_window);
+  gst_object_replace ((GstObject **) old_window_ptr, GST_OBJECT (new_window));
 }
 
 /**
@@ -302,9 +358,9 @@ gst_vaapi_window_replace (GstVaapiWindow ** old_window_ptr,
 GstVaapiDisplay *
 gst_vaapi_window_get_display (GstVaapiWindow * window)
 {
-  g_return_val_if_fail (window != NULL, NULL);
+  g_return_val_if_fail (GST_VAAPI_IS_WINDOW (window), NULL);
 
-  return GST_VAAPI_OBJECT_DISPLAY (window);
+  return GST_VAAPI_WINDOW_DISPLAY (window);
 }
 
 /**
@@ -317,7 +373,7 @@ gst_vaapi_window_get_display (GstVaapiWindow * window)
 void
 gst_vaapi_window_show (GstVaapiWindow * window)
 {
-  g_return_if_fail (window != NULL);
+  g_return_if_fail (GST_VAAPI_IS_WINDOW (window));
 
   GST_VAAPI_WINDOW_GET_CLASS (window)->show (window);
   window->check_geometry = TRUE;
@@ -333,7 +389,7 @@ gst_vaapi_window_show (GstVaapiWindow * window)
 void
 gst_vaapi_window_hide (GstVaapiWindow * window)
 {
-  g_return_if_fail (window != NULL);
+  g_return_if_fail (GST_VAAPI_IS_WINDOW (window));
 
   GST_VAAPI_WINDOW_GET_CLASS (window)->hide (window);
 }
@@ -349,7 +405,7 @@ gst_vaapi_window_hide (GstVaapiWindow * window)
 gboolean
 gst_vaapi_window_get_fullscreen (GstVaapiWindow * window)
 {
-  g_return_val_if_fail (window != NULL, FALSE);
+  g_return_val_if_fail (GST_VAAPI_IS_WINDOW (window), FALSE);
 
   gst_vaapi_window_ensure_size (window);
 
@@ -368,7 +424,7 @@ gst_vaapi_window_set_fullscreen (GstVaapiWindow * window, gboolean fullscreen)
 {
   const GstVaapiWindowClass *klass;
 
-  g_return_if_fail (window != NULL);
+  g_return_if_fail (GST_VAAPI_IS_WINDOW (window));
 
   klass = GST_VAAPI_WINDOW_GET_CLASS (window);
 
@@ -390,7 +446,7 @@ gst_vaapi_window_set_fullscreen (GstVaapiWindow * window, gboolean fullscreen)
 guint
 gst_vaapi_window_get_width (GstVaapiWindow * window)
 {
-  g_return_val_if_fail (window != NULL, 0);
+  g_return_val_if_fail (GST_VAAPI_IS_WINDOW (window), 0);
 
   gst_vaapi_window_ensure_size (window);
 
@@ -408,7 +464,7 @@ gst_vaapi_window_get_width (GstVaapiWindow * window)
 guint
 gst_vaapi_window_get_height (GstVaapiWindow * window)
 {
-  g_return_val_if_fail (window != NULL, 0);
+  g_return_val_if_fail (GST_VAAPI_IS_WINDOW (window), 0);
 
   gst_vaapi_window_ensure_size (window);
 
@@ -427,7 +483,7 @@ void
 gst_vaapi_window_get_size (GstVaapiWindow * window, guint * width_ptr,
     guint * height_ptr)
 {
-  g_return_if_fail (window != NULL);
+  g_return_if_fail (GST_VAAPI_IS_WINDOW (window));
 
   gst_vaapi_window_ensure_size (window);
 
@@ -448,7 +504,7 @@ gst_vaapi_window_get_size (GstVaapiWindow * window, guint * width_ptr,
 void
 gst_vaapi_window_set_width (GstVaapiWindow * window, guint width)
 {
-  g_return_if_fail (window != NULL);
+  g_return_if_fail (GST_VAAPI_IS_WINDOW (window));
 
   gst_vaapi_window_set_size (window, width, window->height);
 }
@@ -463,7 +519,7 @@ gst_vaapi_window_set_width (GstVaapiWindow * window, guint width)
 void
 gst_vaapi_window_set_height (GstVaapiWindow * window, guint height)
 {
-  g_return_if_fail (window != NULL);
+  g_return_if_fail (GST_VAAPI_IS_WINDOW (window));
 
   gst_vaapi_window_set_size (window, window->width, height);
 }
@@ -479,7 +535,7 @@ gst_vaapi_window_set_height (GstVaapiWindow * window, guint height)
 void
 gst_vaapi_window_set_size (GstVaapiWindow * window, guint width, guint height)
 {
-  g_return_if_fail (window != NULL);
+  g_return_if_fail (GST_VAAPI_IS_WINDOW (window));
 
   if (width == window->width && height == window->height)
     return;
@@ -541,7 +597,7 @@ gst_vaapi_window_put_surface (GstVaapiWindow * window,
   const GstVaapiWindowClass *klass;
   GstVaapiRectangle src_rect_default, dst_rect_default;
 
-  g_return_val_if_fail (window != NULL, FALSE);
+  g_return_val_if_fail (GST_VAAPI_IS_WINDOW (window), FALSE);
   g_return_val_if_fail (surface != NULL, FALSE);
 
   klass = GST_VAAPI_WINDOW_GET_CLASS (window);
@@ -596,7 +652,7 @@ gst_vaapi_window_put_pixmap (GstVaapiWindow * window,
   const GstVaapiWindowClass *klass;
   GstVaapiRectangle src_rect_default, dst_rect_default;
 
-  g_return_val_if_fail (window != NULL, FALSE);
+  g_return_val_if_fail (GST_VAAPI_IS_WINDOW (window), FALSE);
   g_return_val_if_fail (pixmap != NULL, FALSE);
 
   klass = GST_VAAPI_WINDOW_GET_CLASS (window);
@@ -625,7 +681,7 @@ gst_vaapi_window_put_pixmap (GstVaapiWindow * window,
 void
 gst_vaapi_window_reconfigure (GstVaapiWindow * window)
 {
-  g_return_if_fail (window != NULL);
+  g_return_if_fail (GST_VAAPI_IS_WINDOW (window));
 
   window->check_geometry = TRUE;
   gst_vaapi_window_ensure_size (window);
@@ -642,7 +698,7 @@ gst_vaapi_window_unblock (GstVaapiWindow * window)
 {
   const GstVaapiWindowClass *klass;
 
-  g_return_val_if_fail (window != NULL, FALSE);
+  g_return_val_if_fail (GST_VAAPI_IS_WINDOW (window), FALSE);
 
   klass = GST_VAAPI_WINDOW_GET_CLASS (window);
 
@@ -663,7 +719,7 @@ gst_vaapi_window_unblock_cancel (GstVaapiWindow * window)
 {
   const GstVaapiWindowClass *klass;
 
-  g_return_val_if_fail (window != NULL, FALSE);
+  g_return_val_if_fail (GST_VAAPI_IS_WINDOW (window), FALSE);
 
   klass = GST_VAAPI_WINDOW_GET_CLASS (window);
 
