@@ -28,7 +28,6 @@
  */
 
 #include "sysdeps.h"
-#include <string.h>
 #include <X11/Xatom.h>
 #include "gstvaapicompat.h"
 #include "gstvaapiwindow_x11.h"
@@ -41,8 +40,14 @@
 #include "gstvaapiutils.h"
 #include "gstvaapiutils_x11.h"
 
-#define DEBUG 1
-#include "gstvaapidebug.h"
+GST_DEBUG_CATEGORY_EXTERN (gst_debug_vaapi_window);
+#define GST_CAT_DEFAULT gst_debug_vaapi_window
+
+#define GST_VAAPI_WINDOW_X11_GET_CLASS(obj) \
+   (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_VAAPI_WINDOW_X11, GstVaapiWindowX11Class))
+
+G_DEFINE_TYPE_WITH_PRIVATE (GstVaapiWindowX11, gst_vaapi_window_x11,
+    GST_TYPE_VAAPI_WINDOW);
 
 #define _NET_WM_STATE_REMOVE    0       /* remove/unset property */
 #define _NET_WM_STATE_ADD       1       /* add/set property      */
@@ -53,13 +58,13 @@ send_wmspec_change_state (GstVaapiWindow * window, Atom state, gboolean add)
 {
   GstVaapiWindowX11Private *const priv =
       GST_VAAPI_WINDOW_X11_GET_PRIVATE (window);
-  Display *const dpy = GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
+  Display *const dpy = GST_VAAPI_WINDOW_NATIVE_DISPLAY (window);
   XClientMessageEvent xclient;
 
   memset (&xclient, 0, sizeof (xclient));
 
   xclient.type = ClientMessage;
-  xclient.window = GST_VAAPI_OBJECT_ID (window);
+  xclient.window = GST_VAAPI_WINDOW_ID (window);
   xclient.message_type = priv->atom_NET_WM_STATE;
   xclient.format = 32;
 
@@ -78,15 +83,15 @@ send_wmspec_change_state (GstVaapiWindow * window, Atom state, gboolean add)
 static void
 wait_event (GstVaapiWindow * window, int type)
 {
-  Display *const dpy = GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
-  const Window xid = GST_VAAPI_OBJECT_ID (window);
+  Display *const dpy = GST_VAAPI_WINDOW_NATIVE_DISPLAY (window);
+  const Window xid = GST_VAAPI_WINDOW_ID (window);
   XEvent e;
   Bool got_event;
 
   for (;;) {
-    GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+    GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
     got_event = XCheckTypedWindowEvent (dpy, xid, type, &e);
-    GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+    GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
     if (got_event)
       break;
     g_usleep (10);
@@ -97,8 +102,8 @@ static gboolean
 timed_wait_event (GstVaapiWindow * window, int type, guint64 end_time,
     XEvent * e)
 {
-  Display *const dpy = GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
-  const Window xid = GST_VAAPI_OBJECT_ID (window);
+  Display *const dpy = GST_VAAPI_WINDOW_NATIVE_DISPLAY (window);
+  const Window xid = GST_VAAPI_WINDOW_ID (window);
   XEvent tmp_event;
   GTimeVal now;
   guint64 now_time;
@@ -107,17 +112,17 @@ timed_wait_event (GstVaapiWindow * window, int type, guint64 end_time,
   if (!e)
     e = &tmp_event;
 
-  GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
   got_event = XCheckTypedWindowEvent (dpy, xid, type, e);
-  GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
   if (got_event)
     return TRUE;
 
   do {
     g_usleep (10);
-    GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+    GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
     got_event = XCheckTypedWindowEvent (dpy, xid, type, e);
-    GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+    GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
     if (got_event)
       return TRUE;
     g_get_current_time (&now);
@@ -131,15 +136,15 @@ gst_vaapi_window_x11_show (GstVaapiWindow * window)
 {
   GstVaapiWindowX11Private *const priv =
       GST_VAAPI_WINDOW_X11_GET_PRIVATE (window);
-  Display *const dpy = GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
-  const Window xid = GST_VAAPI_OBJECT_ID (window);
+  Display *const dpy = GST_VAAPI_WINDOW_NATIVE_DISPLAY (window);
+  const Window xid = GST_VAAPI_WINDOW_ID (window);
   XWindowAttributes wattr;
   gboolean has_errors;
 
   if (priv->is_mapped)
     return TRUE;
 
-  GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
   x11_trap_errors ();
   if (window->use_foreign_window) {
     XGetWindowAttributes (dpy, xid, &wattr);
@@ -148,17 +153,17 @@ gst_vaapi_window_x11_show (GstVaapiWindow * window)
   }
   XMapWindow (dpy, xid);
   has_errors = x11_untrap_errors () != 0;
-  GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
 
   if (!has_errors) {
     wait_event (window, MapNotify);
     if (window->use_foreign_window &&
         !(wattr.your_event_mask & StructureNotifyMask)) {
-      GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+      GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
       x11_trap_errors ();
       XSelectInput (dpy, xid, wattr.your_event_mask);
       has_errors = x11_untrap_errors () != 0;
-      GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+      GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
     }
     priv->is_mapped = TRUE;
 
@@ -173,15 +178,15 @@ gst_vaapi_window_x11_hide (GstVaapiWindow * window)
 {
   GstVaapiWindowX11Private *const priv =
       GST_VAAPI_WINDOW_X11_GET_PRIVATE (window);
-  Display *const dpy = GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
-  const Window xid = GST_VAAPI_OBJECT_ID (window);
+  Display *const dpy = GST_VAAPI_WINDOW_NATIVE_DISPLAY (window);
+  const Window xid = GST_VAAPI_WINDOW_ID (window);
   XWindowAttributes wattr;
   gboolean has_errors;
 
   if (!priv->is_mapped)
     return TRUE;
 
-  GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
   x11_trap_errors ();
   if (window->use_foreign_window) {
     XGetWindowAttributes (dpy, xid, &wattr);
@@ -190,17 +195,17 @@ gst_vaapi_window_x11_hide (GstVaapiWindow * window)
   }
   XUnmapWindow (dpy, xid);
   has_errors = x11_untrap_errors () != 0;
-  GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
 
   if (!has_errors) {
     wait_event (window, UnmapNotify);
     if (window->use_foreign_window &&
         !(wattr.your_event_mask & StructureNotifyMask)) {
-      GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+      GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
       x11_trap_errors ();
       XSelectInput (dpy, xid, wattr.your_event_mask);
       has_errors = x11_untrap_errors () != 0;
-      GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+      GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
     }
     priv->is_mapped = FALSE;
   }
@@ -213,9 +218,9 @@ gst_vaapi_window_x11_create (GstVaapiWindow * window, guint * width,
 {
   GstVaapiWindowX11Private *const priv =
       GST_VAAPI_WINDOW_X11_GET_PRIVATE (window);
-  GstVaapiDisplay *const display = GST_VAAPI_OBJECT_DISPLAY (window);
-  Display *const dpy = GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
-  Window xid = GST_VAAPI_OBJECT_ID (window);
+  GstVaapiDisplay *const display = GST_VAAPI_WINDOW_DISPLAY (window);
+  Display *const dpy = GST_VAAPI_WINDOW_NATIVE_DISPLAY (window);
+  Window xid = GST_VAAPI_WINDOW_ID (window);
   guint vid = 0;
   Colormap cmap = None;
   const GstVaapiDisplayClass *display_class;
@@ -230,14 +235,14 @@ gst_vaapi_window_x11_create (GstVaapiWindow * window, guint * width,
   };
 
   priv->has_xrender =
-      GST_VAAPI_DISPLAY_HAS_XRENDER (GST_VAAPI_OBJECT_DISPLAY (window));
+      GST_VAAPI_DISPLAY_HAS_XRENDER (GST_VAAPI_WINDOW_DISPLAY (window));
 
   if (window->use_foreign_window && xid) {
-    GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+    GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
     XGetWindowAttributes (dpy, xid, &wattr);
     priv->is_mapped = wattr.map_state == IsViewable;
     ok = x11_get_geometry (dpy, xid, NULL, NULL, width, height, NULL);
-    GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+    GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
     return ok;
   }
 
@@ -257,7 +262,7 @@ gst_vaapi_window_x11_create (GstVaapiWindow * window, guint * width,
       cmap = window_class->get_colormap (window);
   }
 
-  GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
   XInternAtoms (dpy,
       (char **) atom_names, G_N_ELEMENTS (atom_names), False, atoms);
   priv->atom_NET_WM_STATE = atoms[0];
@@ -266,54 +271,54 @@ gst_vaapi_window_x11_create (GstVaapiWindow * window, guint * width,
   xid = x11_create_window (dpy, *width, *height, vid, cmap);
   if (xid)
     XRaiseWindow (dpy, xid);
-  GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
 
   GST_DEBUG ("xid %" GST_VAAPI_ID_FORMAT, GST_VAAPI_ID_ARGS (xid));
-  GST_VAAPI_OBJECT_ID (window) = xid;
+  GST_VAAPI_WINDOW_ID (window) = xid;
   return xid != None;
 }
 
 static void
-gst_vaapi_window_x11_destroy (GstVaapiWindow * window)
+gst_vaapi_window_x11_finalize (GObject * object)
 {
-  Display *const dpy = GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
-  const Window xid = GST_VAAPI_OBJECT_ID (window);
+  GstVaapiWindow *window = GST_VAAPI_WINDOW (object);
+  Display *const dpy = GST_VAAPI_WINDOW_NATIVE_DISPLAY (window);
+  const Window xid = GST_VAAPI_WINDOW_ID (window);
 
-#ifdef HAVE_XRENDER
+#if HAVE_XRENDER
   GstVaapiWindowX11Private *const priv =
       GST_VAAPI_WINDOW_X11_GET_PRIVATE (window);
   if (priv->picture) {
-    GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+    GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
     XRenderFreePicture (dpy, priv->picture);
-    GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+    GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
     priv->picture = None;
   }
 #endif
 
   if (xid) {
     if (!window->use_foreign_window) {
-      GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+      GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
       XDestroyWindow (dpy, xid);
-      GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+      GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
     }
-    GST_VAAPI_OBJECT_ID (window) = None;
+    GST_VAAPI_WINDOW_ID (window) = None;
   }
 
-  GST_VAAPI_WINDOW_X11_GET_CLASS (window)->parent_finalize (GST_VAAPI_OBJECT
-      (window));
+  G_OBJECT_CLASS (gst_vaapi_window_x11_parent_class)->finalize (object);
 }
 
 static gboolean
 gst_vaapi_window_x11_get_geometry (GstVaapiWindow * window,
     gint * px, gint * py, guint * pwidth, guint * pheight)
 {
-  Display *const dpy = GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
-  const Window xid = GST_VAAPI_OBJECT_ID (window);
+  Display *const dpy = GST_VAAPI_WINDOW_NATIVE_DISPLAY (window);
+  const Window xid = GST_VAAPI_WINDOW_ID (window);
   gboolean success;
 
-  GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
   success = x11_get_geometry (dpy, xid, px, py, pwidth, pheight, NULL);
-  GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
   return success;
 }
 
@@ -323,15 +328,15 @@ gst_vaapi_window_x11_set_fullscreen (GstVaapiWindow * window,
 {
   GstVaapiWindowX11Private *const priv =
       GST_VAAPI_WINDOW_X11_GET_PRIVATE (window);
-  Display *const dpy = GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
-  const Window xid = GST_VAAPI_OBJECT_ID (window);
+  Display *const dpy = GST_VAAPI_WINDOW_NATIVE_DISPLAY (window);
+  const Window xid = GST_VAAPI_WINDOW_ID (window);
   XEvent e;
   guint width, height;
   gboolean has_errors;
   GTimeVal now;
   guint64 end_time;
 
-  GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
   x11_trap_errors ();
   if (fullscreen) {
     if (!priv->is_mapped) {
@@ -358,7 +363,7 @@ gst_vaapi_window_x11_set_fullscreen (GstVaapiWindow * window,
   }
   XSync (dpy, False);
   has_errors = x11_untrap_errors () != 0;
-  GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
   if (has_errors)
     return FALSE;
 
@@ -369,7 +374,7 @@ gst_vaapi_window_x11_set_fullscreen (GstVaapiWindow * window,
     end_time = DELAY + ((guint64) now.tv_sec * 1000000 + now.tv_usec);
     while (timed_wait_event (window, ConfigureNotify, end_time, &e)) {
       if (fullscreen) {
-        gst_vaapi_display_get_size (GST_VAAPI_OBJECT_DISPLAY (window),
+        gst_vaapi_display_get_size (GST_VAAPI_WINDOW_DISPLAY (window),
             &width, &height);
         if (e.xconfigure.width == width && e.xconfigure.height == height)
           return TRUE;
@@ -388,15 +393,15 @@ gst_vaapi_window_x11_resize (GstVaapiWindow * window, guint width, guint height)
 {
   gboolean has_errors;
 
-  if (!GST_VAAPI_OBJECT_ID (window))
+  if (!GST_VAAPI_WINDOW_ID (window))
     return FALSE;
 
-  GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
   x11_trap_errors ();
-  XResizeWindow (GST_VAAPI_OBJECT_NATIVE_DISPLAY (window),
-      GST_VAAPI_OBJECT_ID (window), width, height);
+  XResizeWindow (GST_VAAPI_WINDOW_NATIVE_DISPLAY (window),
+      GST_VAAPI_WINDOW_ID (window), width, height);
   has_errors = x11_untrap_errors () != 0;
-  GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
   return !has_errors;
 }
 
@@ -408,10 +413,10 @@ gst_vaapi_window_x11_put_surface (GstVaapiWindow * window,
 {
   VAStatus status;
 
-  GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
-  status = vaPutSurface (GST_VAAPI_OBJECT_VADISPLAY (window),
+  GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
+  status = vaPutSurface (GST_VAAPI_WINDOW_VADISPLAY (window),
       surface_id,
-      GST_VAAPI_OBJECT_ID (window),
+      GST_VAAPI_WINDOW_ID (window),
       src_rect->x,
       src_rect->y,
       src_rect->width,
@@ -422,7 +427,7 @@ gst_vaapi_window_x11_put_surface (GstVaapiWindow * window,
       dst_rect->height, NULL, 0, from_GstVaapiSurfaceRenderFlags (flags)
       );
 
-  GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
 
   return status;
 }
@@ -496,11 +501,11 @@ gst_vaapi_window_x11_render_pixmap_xrender (GstVaapiWindow * window,
     GstVaapiPixmap * pixmap,
     const GstVaapiRectangle * src_rect, const GstVaapiRectangle * dst_rect)
 {
-#ifdef HAVE_XRENDER
+#if HAVE_XRENDER
   GstVaapiWindowX11Private *const priv =
       GST_VAAPI_WINDOW_X11_GET_PRIVATE (window);
-  Display *const dpy = GST_VAAPI_OBJECT_NATIVE_DISPLAY (window);
-  const Window win = GST_VAAPI_OBJECT_ID (window);
+  Display *const dpy = GST_VAAPI_WINDOW_NATIVE_DISPLAY (window);
+  const Window win = GST_VAAPI_WINDOW_ID (window);
   const Pixmap pix = GST_VAAPI_OBJECT_ID (pixmap);
   Picture picture;
   XRenderPictFormat *pic_fmt;
@@ -510,12 +515,12 @@ gst_vaapi_window_x11_render_pixmap_xrender (GstVaapiWindow * window,
 
   /* Ensure Picture for window is created */
   if (!priv->picture) {
-    GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+    GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
     XGetWindowAttributes (dpy, win, &wattr);
     pic_fmt = XRenderFindVisualFormat (dpy, wattr.visual);
     if (pic_fmt)
       priv->picture = XRenderCreatePicture (dpy, win, pic_fmt, 0, NULL);
-    GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+    GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
     if (!priv->picture)
       return FALSE;
   }
@@ -530,9 +535,9 @@ gst_vaapi_window_x11_render_pixmap_xrender (GstVaapiWindow * window,
       fmt = PictStandardARGB32;
       op = PictOpOver;
     get_pic_fmt:
-      GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+      GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
       pic_fmt = XRenderFindStandardFormat (dpy, fmt);
-      GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+      GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
       break;
     default:
       pic_fmt = NULL;
@@ -541,7 +546,7 @@ gst_vaapi_window_x11_render_pixmap_xrender (GstVaapiWindow * window,
   if (!pic_fmt)
     return FALSE;
 
-  GST_VAAPI_OBJECT_LOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_LOCK_DISPLAY (window);
   do {
     const double sx = (double) src_rect->width / dst_rect->width;
     const double sy = (double) src_rect->height / dst_rect->height;
@@ -570,7 +575,7 @@ gst_vaapi_window_x11_render_pixmap_xrender (GstVaapiWindow * window,
   } while (0);
   if (picture)
     XRenderFreePicture (dpy, picture);
-  GST_VAAPI_OBJECT_UNLOCK_DISPLAY (window);
+  GST_VAAPI_WINDOW_UNLOCK_DISPLAY (window);
   return success;
 #endif
   return FALSE;
@@ -592,17 +597,13 @@ gst_vaapi_window_x11_render_pixmap (GstVaapiWindow * window,
   return FALSE;
 }
 
-void
+static void
 gst_vaapi_window_x11_class_init (GstVaapiWindowX11Class * klass)
 {
-  GstVaapiObjectClass *const object_class = GST_VAAPI_OBJECT_CLASS (klass);
+  GObjectClass *const object_class = G_OBJECT_CLASS (klass);
   GstVaapiWindowClass *const window_class = GST_VAAPI_WINDOW_CLASS (klass);
 
-  gst_vaapi_window_class_init (&klass->parent_class);
-
-  klass->parent_finalize = object_class->finalize;
-  object_class->finalize = (GstVaapiObjectFinalizeFunc)
-      gst_vaapi_window_x11_destroy;
+  object_class->finalize = gst_vaapi_window_x11_finalize;
 
   window_class->create = gst_vaapi_window_x11_create;
   window_class->show = gst_vaapi_window_x11_show;
@@ -614,11 +615,10 @@ gst_vaapi_window_x11_class_init (GstVaapiWindowX11Class * klass)
   window_class->render_pixmap = gst_vaapi_window_x11_render_pixmap;
 }
 
-#define gst_vaapi_window_x11_finalize \
-    gst_vaapi_window_x11_destroy
-
-GST_VAAPI_OBJECT_DEFINE_CLASS_WITH_CODE (GstVaapiWindowX11,
-    gst_vaapi_window_x11, gst_vaapi_window_x11_class_init (&g_class));
+static void
+gst_vaapi_window_x11_init (GstVaapiWindowX11 * window)
+{
+}
 
 /**
  * gst_vaapi_window_x11_new:
@@ -635,14 +635,10 @@ GST_VAAPI_OBJECT_DEFINE_CLASS_WITH_CODE (GstVaapiWindowX11,
 GstVaapiWindow *
 gst_vaapi_window_x11_new (GstVaapiDisplay * display, guint width, guint height)
 {
-  GST_DEBUG ("new window, size %ux%u", width, height);
-
   g_return_val_if_fail (GST_VAAPI_IS_DISPLAY_X11 (display), NULL);
 
-  return
-      gst_vaapi_window_new_internal (GST_VAAPI_WINDOW_CLASS
-      (gst_vaapi_window_x11_class ()), display, GST_VAAPI_ID_INVALID, width,
-      height);
+  return gst_vaapi_window_new_internal (GST_TYPE_VAAPI_WINDOW_X11, display,
+      GST_VAAPI_ID_INVALID, width, height);
 }
 
 /**
@@ -660,13 +656,11 @@ gst_vaapi_window_x11_new (GstVaapiDisplay * display, guint width, guint height)
 GstVaapiWindow *
 gst_vaapi_window_x11_new_with_xid (GstVaapiDisplay * display, Window xid)
 {
-  GST_DEBUG ("new window from xid 0x%08x", (guint) xid);
-
   g_return_val_if_fail (GST_VAAPI_IS_DISPLAY_X11 (display), NULL);
   g_return_val_if_fail (xid != None, NULL);
 
-  return gst_vaapi_window_new_internal (GST_VAAPI_WINDOW_CLASS
-      (gst_vaapi_window_x11_class ()), display, xid, 0, 0);
+  return gst_vaapi_window_new_internal (GST_TYPE_VAAPI_WINDOW_X11, display,
+      xid, 0, 0);
 }
 
 /**
@@ -682,9 +676,9 @@ gst_vaapi_window_x11_new_with_xid (GstVaapiDisplay * display, Window xid)
 Window
 gst_vaapi_window_x11_get_xid (GstVaapiWindowX11 * window)
 {
-  g_return_val_if_fail (window != NULL, None);
+  g_return_val_if_fail (GST_VAAPI_IS_WINDOW_X11 (window), None);
 
-  return GST_VAAPI_OBJECT_ID (window);
+  return GST_VAAPI_WINDOW_ID (window);
 }
 
 /**
@@ -699,7 +693,7 @@ gst_vaapi_window_x11_get_xid (GstVaapiWindowX11 * window)
 gboolean
 gst_vaapi_window_x11_is_foreign_xid (GstVaapiWindowX11 * window)
 {
-  g_return_val_if_fail (window != NULL, FALSE);
+  g_return_val_if_fail (GST_VAAPI_IS_WINDOW_X11 (window), FALSE);
 
   return GST_VAAPI_WINDOW (window)->use_foreign_window;
 }
